/*
 * This file is part of the Pablo Performance Analysis Environment
 *
 *                                           TM
 * The Pablo Performance Analysis Environment   software is *not* in
 * the public domain.  However, it is freely available without fee for
 * education, research, and non-profit purposes.  By obtaining copies
 * of this and other files that comprise the Pablo Performance Analysis
 * Environment, you, the Licensee, agree to abide by the following
 * conditions and understandings with respect to the copyrighted software:
 * 
 * 1.  The software is copyrighted in the name of the Board of Trustees
 *     of the University of Illinois (UI), and ownership of the software
 *     remains with the UI. 
 *
 * 2.  Permission to use, copy, and modify this software and its documentation
 *     for education, research, and non-profit purposes is hereby granted
 *     to Licensee, provided that the copyright notice, the original author's
 *     names and unit identification, and this permission notice appear on
 *     all such copies, and that no charge be made for such copies.  Any
 *     entity desiring permission to incorporate this software into commercial
 *     products should contact:
 *
 *          Professor Daniel A. Reed                 reed@cs.uiuc.edu
 *          University of Illinois
 *          Department of Computer Science
 *          2413 Digital Computer Laboratory
 *          1304 West Springfield Avenue
 *          Urbana, Illinois  61801
 *          USA
 *
 * 3.  Licensee may not use the name, logo, or any other symbol of the UI
 *     nor the names of any of its employees nor any adaptation thereof in
 *     advertizing or publicity pertaining to the software without specific
 *     prior written approval of the UI.
 *
 * 4.  THE UI MAKES NO REPRESENTATIONS ABOUT THE SUITABILITY OF THE
 *     SOFTWARE FOR ANY PURPOSE.  IT IS PROVIDED "AS IS" WITHOUT EXPRESS
 *     OR IMPLIED WARRANTY.
 *
 * 5.  The UI shall not be liable for any damages suffered by Licensee from
 *     the use of this software.
 *
 * 6.  The software was developed under agreements between the UI and the
 *     Federal Government which entitle the Government to certain rights.
 *
 **************************************************************************
 *
 * Developed by: The TAPESTRY Parallel Computing Laboratory
 *		 University of Illinois at Urbana-Champaign
 *		 Department of Computer Science
 *		 1304 W. Springfield Avenue
 *		 Urbana, IL	61801
 *
 * Copyright (c) 1987-1994
 * The University of Illinois Board of Trustees.
 *	All Rights Reserved.
 *
 * Author: Daniel A. Reed (reed@cs.uiuc.edu)
 * Project Manager and Principal Investigator:
 *	Daniel A. Reed (reed@cs.uiuc.edu)
 *
 * Funded by: National Science Foundation grants NSF CCR86-57696,
 * NSF CCR87-06653 and NSF CDA87-22836 (Tapestry), NASA ICLASS Contract
 * No. NAG-1-613, DARPA Contract No. DABT63-91-K-0004, by a grant
 * from the Digital Equipment Corporation External Research Program,
 * and by a collaborative research agreement with the Intel Supercomputer
 * Systems Division.
 *
 */
/*
 * Scatter3DWrapper.h: A class wrapper for the Scatter3D performance widget
 *
 *	$Header: /mnt/Pablo-guitar/Stable.2-94/Visual/Src/Widgets/Wrappers/RCS/Scatter3DWrapper.h,v 1.10 1994/02/25 04:44:49 aydt Exp $
 */


#ifndef Scatter3DWrapper_h
#define Scatter3DWrapper_h

#include "PerfWidget.h"
#include "DisplayColorTables.h"  /* Maintains widget color table information */

typedef enum {L_X, L_Y, L_Z } scatterLabelType;


class Scatter3DWrapper : public PerfWidget {
/*
*
* A Scatter3DWrapper is the C++ wrapper for the 3D scatter plot display widget.
*
*/
private:
        DisplayColorTables colorTable;
	Widget xLabel, yLabel, zLabel;
	WidgetList childList;

protected:

	/* Method _alignDataArea:  Adjust the scatter position based on
	                           the existence of labels */
	void _alignDataArea();

public:

	/* Method Scatter3DWrapper: 	the Scatter3DWrapper constructor  */
	Scatter3DWrapper( WidgetBase *par, const XArgs& args,
			const CString& );

	/* Method ~Scatter3DWrapper: 	the Scatter3DWrapper destructor	 */
	~Scatter3DWrapper();

	/* Method getScatterLabel:  retrieve the given label char string */
	void getScatterLabel( int whichLabel, char **aLabel ) const;

	/* Method setScatterLabel:  define a label string for one of the
	                            scatter label */
	void setScatterLabel( int whichLabel, const char *stringLabel );

	/* Method get/setScatterAttr:  set X attributes for the scatter 
	                               (not its form) */
	void getScatterAttr( String name, XtArgVal value );
	void setScatterAttr( String name, XtArgVal value );

	/* Method setPerfWidgetValues:	sets all values of the scatter
	*				plot display widget		*/
	void setPerfWidgetValues( int *xValues, int *yValues,
				  int *zValues, int pointCount,
				  int setNumber );

	/* Method setPerfWidgetColors:	sets the colors for the
	*				scatter plot widget		*/
	void setPerfWidgetColors();

};

#endif
